@@ -725,6 +725,11 @@ func (b *Backend) handleRotation(ctx context.Context, req *logical.Request) (*lo
 		return nil, logical.ErrUnsupportedOperation
 	}
 
+	// rotation is a write operation, so we short-circuit the request
+	if !b.WriteSafeReplicationState() {
+		return nil, logical.ErrReadOnly
+	}
+
 	err := b.RotateCredential(ctx, req)
 	if err != nil {
 		return nil, err
@@ -818,6 +823,41 @@ func (b *Backend) RecordObservation(ctx context.Context, observationType string,
 	return b.observations.RecordObservationFromPlugin(ctx, observationType, data)
 }
 
+// RecordObservationWithRequest is used to record observations through the
+// plugin's observation system. It attaches information from the request to the
+// observation data. The request path, client ID, entity ID, and request ID are
+// included.
+// This method returns ErrNoObservations if the observation system has not been
+// configured or enabled.
+func (b *Backend) RecordObservationWithRequest(ctx context.Context, req *logical.Request, observationType string, data map[string]interface{}) error {
+	if b.observations == nil {
+		return ErrNoObservations
+	}
+
+	// Attach request information to the observation data
+	if req != nil {
+		if data == nil {
+			data = make(map[string]interface{})
+		}
+		data["path"] = req.Path
+		data["client_id"] = req.ClientID
+		data["entity_id"] = req.EntityID
+		data["request_id"] = req.ID
+	}
+
+	return b.observations.RecordObservationFromPlugin(ctx, observationType, data)
+}
+
+// TryRecordObservationWithRequest is like RecordObservationWithRequest but
+// logs a warning instead of returning an error if recording the observation fails
+// for reasons other than the observation system not being configured/enabled
+func (b *Backend) TryRecordObservationWithRequest(ctx context.Context, req *logical.Request, observationType string, data map[string]interface{}) {
+	err := b.RecordObservationWithRequest(ctx, req, observationType, data)
+	if err != nil && !errors.Is(err, ErrNoObservations) {
+		b.Logger().Warn("failed to record observation", "error", err, "observation_type", observationType)
+	}
+}
+
 // FieldSchema is a basic schema to describe the format of a path field.
 type FieldSchema struct {
 	Type        FieldType
